-------------------------------
-- Akaizer wrapper tool v1.4 --
-------------------------------

-- constants --

local MIN_TIME_FACTOR = 25
local DEF_TIME_FACTOR = 100
local MAX_TIME_FACTOR = 2000
local MIN_CYCLE_LENGTH = 20
local DEF_CYCLE_LENGTH = 1000
local MAX_CYCLE_LENGTH = 2000
local MIN_TRANSPOSE = -24
local DEF_TRANSPOSE = 0
local MAX_TRANSPOSE = 24
local DEF_ALGORITHM = 1 -- 1=Revised 2=Classic
local MINIMAL_LENGTH = 40 -- milliseconds
local BIN = {
  WINDOWS = "bin/windows/Akaizer.exe",
  LINUX = "bin/linux/Akaizer",
  MACINTOSH = "bin/macos/Akaizer"
}

-- variables --

local last_time_factor = DEF_TIME_FACTOR
local last_cycle_length = DEF_CYCLE_LENGTH
local last_transpose = DEF_TRANSPOSE
local last_algorithm = DEF_ALGORITHM
local show_dialog = nil

-- init --

if(os.platform() == "LINUX" or os.platform() == "MACINTOSH") then
  io.chmod(string.format("%q", renoise.tool().bundle_path..BIN[os.platform()]), 755)
end

-- menu entries --

renoise.tool():add_menu_entry {
  name = "Sample Editor:Process:Akaizer...",
  invoke = function()
    show_dialog()
  end
}

-- key binding --

renoise.tool():add_keybinding {
  name = "Sample Editor:Process:Akaizer",
  invoke = function()
    show_dialog()
  end
}

-- helpers --

local function remove_file_extension(filename)
  local found, len, remainder = filename:find("^(.*)%.[^%.]*$")
  
  if found then
    return remainder
  else
    return filename
  end
end

local function parse_path(path)
  local dir, name, ext = path:match("(.-)([^\\/]-%.?([^%.\\/]*))$")
  name = remove_file_extension(name)
  
  return dir, name
end

local function get_sample_length() -- in seconds
  local sample = renoise.song().selected_sample.sample_buffer
  
  local sample_rate = sample.sample_rate
  local frames = sample.number_of_frames
  
  return frames / sample_rate
end

-- processing --

local function process_sample(time_factor, cycle_length, transpose, is_classic)
  if is_classic then
    time_factor = math.floor(time_factor)
  end
  if time_factor == DEF_TIME_FACTOR and transpose == DEF_TRANSPOSE then
    renoise.app():show_message("Sample wasn't processed because you didn't change Time Factor or Transpose.")
    show_dialog()
  else
    local tmp_path = os.tmpname("wav")
    local tmp_dir, tmp_name = parse_path(tmp_path)
    
    local output = string.format("%s-%s%%_%u_%s%d_%s.wav",
      tmp_name,
      time_factor,
      cycle_length,
      transpose > 0 and "+" or "",
      transpose,
      is_classic and "C" or "R"
    )
    
    local exe = string.format("%q %s %s %u %d%s",
      renoise.tool().bundle_path..BIN[os.platform()],
      tmp_path,
      time_factor,
      cycle_length,
      transpose,
      is_classic and " -c" or ""
    )
    
    renoise.song().selected_sample.sample_buffer:save_as(tmp_path, "wav")
    
    if os.execute(exe) ~= 0 then
      renoise.app():show_error("Process did not complete successfully...")
      os.remove(tmp_path)
      return
    end
    
    renoise.song().selected_sample.sample_buffer:load_from(tmp_dir..output)
    
    os.remove(tmp_path)
    os.remove(tmp_dir..output)
    
    renoise.app():show_status("Successfully processed!")
  end
end

-- dialogs --

show_dialog = function()
  if renoise.song().selected_sample then
    if get_sample_length() * 1000 < MINIMAL_LENGTH then
      renoise.app():show_error("The sample is too short! Minimal length is "..MINIMAL_LENGTH.." ms.")
      return
    end
  else
    return
  end
  
  local DIALOG_MARGIN = renoise.ViewBuilder.DEFAULT_DIALOG_MARGIN
  local CONTENT_SPACING = renoise.ViewBuilder.DEFAULT_CONTROL_SPACING
  local vb = renoise.ViewBuilder()
  local algorithm_switch = nil
  
  -- view elements --
  
  local time_factor_box = vb:valuebox {
    tooltip = "TIME FACTOR [Min: "..MIN_TIME_FACTOR.."] [Max: "..MAX_TIME_FACTOR.."] [Default: "..DEF_TIME_FACTOR.."]",
    min = MIN_TIME_FACTOR,
    max = MAX_TIME_FACTOR,
    value = last_time_factor,
    width = 80,
    tostring = function(value)
      if algorithm_switch.value == 1 then
        return string.format("%.2f", value)
      else
        return string.format("%u", value)
      end
    end,
    tonumber = function(value)
      if not tonumber(value) then
        return nil
      end
      if algorithm_switch.value == 1 then
        return tonumber(string.format("%.2f", value))
      else
        return tonumber(string.format("%u", value))
      end
    end
  }
  
  local time_factor_button = vb:button {
    tooltip = "Reset TIME FACTOR to default value ["..DEF_TIME_FACTOR.."]",
    text = "^",
    notifier = function()
      time_factor_box.value = DEF_TIME_FACTOR
    end
  }
  
  local cycle_length_box = vb:valuebox {
    tooltip = "CYCLE LENGTH [Min: "..MIN_CYCLE_LENGTH.."] [Max: "..MAX_CYCLE_LENGTH.."] [Default: "..DEF_CYCLE_LENGTH.."]",
    min = MIN_CYCLE_LENGTH,
    max = MAX_CYCLE_LENGTH,
    value = last_cycle_length,
    width = 80
  }
  
  local cycle_length_button = vb:button {
    tooltip = "Reset CYCLE LENGTH to default value ["..DEF_CYCLE_LENGTH.."]",
    text = "^",
    notifier = function()
      cycle_length_box.value = DEF_CYCLE_LENGTH
    end
  }
  
  local transpose_box = vb:valuebox {
    tooltip= "TRANSPOSE [Min: "..MIN_TRANSPOSE.."] [Max: +"..MAX_TRANSPOSE.."] [Default: "..DEF_TRANSPOSE.."]",
    min = MIN_TRANSPOSE,
    max = MAX_TRANSPOSE,
    value = last_transpose,
    width = 80,
    tostring = function(value)
      if value > 0 then
        return string.format("%+d", value)
      else
        return string.format("%d", value)
      end
    end,
    tonumber = function(value)
      if not tonumber(value) then
        return nil
      end
      return tonumber(value)
    end
  }
  
  local transpose_button = vb:button {
    tooltip = "Reset TRANSPOSE to default value ["..DEF_TRANSPOSE.."]",
    text = "^",
    notifier = function()
      transpose_box.value = DEF_TRANSPOSE
    end
  }
  
  algorithm_switch = vb:switch {
    tooltip = "(R)EVISED or (C)LASSIC cyclic algorithm",
    items = {"R", "C"},
    value = last_algorithm,
    width = 80,
    notifier = function()
      -- refresh time factor valuebox by changing its value
      if time_factor_box.value >= MAX_TIME_FACTOR - 1 then
        time_factor_box.value = time_factor_box.value - 1
        time_factor_box.value = time_factor_box.value + 1
      else
        time_factor_box.value = time_factor_box.value + 1
        time_factor_box.value = time_factor_box.value - 1
      end
    end
  }
  
  -- rows --
  
  local time_factor_row = vb:row {
    vb:text {
      text = "Time Factor",
      width = 80
    },
    time_factor_box,
    vb:text {
      text = "%",
      width = 70
    },
    time_factor_button
  }
  
  local cycle_length_row = vb:row {
    vb:text {
      text = "Cycle Length",
      width = 80
    },
    cycle_length_box,
    vb:text {
      text = "samples",
      width = 70
    },
    cycle_length_button
  }
  
  local transpose_row = vb:row {
    vb:text {
      text = "Transpose",
      width = 80
    },
    transpose_box,
    vb:text {
      text = "semitones",
      width = 70
    },
    transpose_button
  }
  
  local algorithm_row = vb:row {
    vb:text {
      text = "Algorithm",
      width = 80
    },
    algorithm_switch
  }
  
  -- prompt --
  
  local prompt = renoise.app():show_custom_prompt(
    "Akaizer",
    
    vb:column {
      margin = DIALOG_MARGIN,
      spacing = CONTENT_SPACING,
      
      time_factor_row,
      cycle_length_row,
      transpose_row,
      algorithm_row
    },
    
    { "Process", "Cancel" }
  )
  
  -- remember current values
  last_time_factor = time_factor_box.value
  last_cycle_length = cycle_length_box.value
  last_transpose = transpose_box.value
  last_algorithm = algorithm_switch.value
  
  if prompt == "Process" then
    process_sample(
      time_factor_box.value,
      cycle_length_box.value, 
      transpose_box.value, 
      algorithm_switch.value == 2 and true or false
    )
  end
end
